#----------------------------------------------------------------------------------------
# Wednesday, January 21, 2004 at 18:25 MS BR
# the first derivatives squares have benn used here
JSU <- function (mu.link="identity", sigma.link="log", nu.link ="identity", tau.link="log")
{
    mstats <- checklink(   "mu.link", "Johnson SU", substitute(mu.link), 
                           c("1/mu^2", "log", "identity"))
    dstats <- checklink("sigma.link", "Johnson SU", substitute(sigma.link), 
                           c("inverse", "log", "identity"))
    vstats <- checklink(   "nu.link", "Johnson SU", substitute(nu.link),    
                           c("1/nu^2", "log", "identity"))
    tstats <- checklink(  "tau.link", "Johnson SU", substitute(tau.link),   
                           c("1/tau^2", "log", "identity")) 
    structure(
          list(family = c("JSU", "Johnson SU"),
           parameters = list(mu=TRUE, sigma=TRUE, nu=TRUE, tau=TRUE), 
                nopar = 4, 
                 type = "Continuous",
              mu.link = as.character(substitute(mu.link)),  
           sigma.link = as.character(substitute(sigma.link)), 
              nu.link = as.character(substitute(nu.link)), 
             tau.link = as.character(substitute(tau.link)), 
           mu.linkfun = mstats$linkfun, 
        sigma.linkfun = dstats$linkfun, 
           nu.linkfun = vstats$linkfun,
           tau.linkfun = tstats$linkfun,  
           mu.linkinv = mstats$linkinv, 
        sigma.linkinv = dstats$linkinv,
           nu.linkinv = vstats$linkinv,
           tau.linkinv = tstats$linkinv, 
                mu.dr = mstats$mu.eta, 
             sigma.dr = dstats$mu.eta, 
                nu.dr = vstats$mu.eta,
               tau.dr = tstats$mu.eta, 
    dldm = function() { 
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
   dldm <- (z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))/(c*sigma)
                      },
   d2ldm2 = function(){
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
   dldm <- (z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))/(c*sigma)
   d2ldm2 <- -dldm*dldm
                      },     
   dldd = function() {  
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dldd <- (z+w^(0.5)*sinh(omega))*(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))
      dldd <- (dldd-1)/sigma                      
      } ,
   d2ldd2 = function(){
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dldd <- (z+w^(0.5)*sinh(omega))*(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))
      dldd <- (dldd-1)/sigma     
      d2ldd2 <- -dldd*dldd
                      },   
     dldv = function() { 
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dlogcdv <- (tau*w*sinh(2*omega))/(w*cosh(2*omega)+1)
      dzdv <- -(z+w^(.5)*sinh(omega))*dlogcdv+(tau*w^(.5)*cosh(omega))
      dldv <- -dlogcdv-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdv+r
                        } ,
    d2ldv2 = function() { 
     w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dlogcdv <- (tau*w*sinh(2*omega))/(w*cosh(2*omega)+1)
      dzdv <- -(z+w^(.5)*sinh(omega))*dlogcdv+(tau*w^(.5)*cosh(omega))
      dldv <- -dlogcdv-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdv+r
      d2ldv2 <-  -dldv*dldv             
                        },
      dldt = function() {
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dlogcdt <- -tau*w*((1/(w-1))+((cosh(2*omega))/(w*cosh(2*omega)+1)))
      dlogcdt <- dlogcdt+((nu*w*sinh(2*omega))/(w*cosh(2*omega)+1))
      dzdt <- -(z+w^(.5)*sinh(omega))*dlogcdt-tau*w^(.5)*sinh(omega)+nu*w^(.5)*cosh(omega)  
      dldt <- -dlogcdt-(1/tau)-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdt+(r*(r+nu))/tau
                        } ,
      d2ldt2 = function() { 
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
      dlogcdt <- -tau*w*((1/(w-1))+((cosh(2*omega))/(w*cosh(2*omega)+1)))
      dlogcdt <- dlogcdt+((nu*w*sinh(2*omega))/(w*cosh(2*omega)+1))
      dzdt <- -(z+w^(.5)*sinh(omega))*dlogcdt-tau*w^(.5)*sinh(omega)+nu*w^(.5)*cosh(omega)  
      dldt <- -dlogcdt-(1/tau)-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdt+(r*(r+nu))/tau
      d2ldt2 <-   -dldt*dldt   
                            } ,
  d2ldmdd = function() {
       w <- exp(tau^2)
   omega <- -nu*tau 
       c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
       z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
       r <- -nu + asinh(z)/tau
    dldm <- (z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))/(c*sigma)
    dldd <- (z+w^(0.5)*sinh(omega))*(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))
    dldd <- (dldd-1)/sigma  
 d2ldmdd <- -(dldm*dldd)
                       },
  d2ldmdv = function() {
       w <- exp(tau^2)
   omega <- -nu*tau 
       c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
       z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
       r <- -nu + asinh(z)/tau
    dldm <- (z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))/(c*sigma)
 dlogcdv <- (tau*w*sinh(2*omega))/(w*cosh(2*omega)+1)
    dzdv <- -(z+w^(.5)*sinh(omega))*dlogcdv+(tau*w^(.5)*cosh(omega))
    dldv <- -dlogcdv-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdv+r   
 d2ldmdv <- -(dldm*dldv)
                       },
  d2ldmdt = function() {
       w <- exp(tau^2)
   omega <- -nu*tau 
       c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
       z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
       r <- -nu + asinh(z)/tau
    dldm <- (z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))/(c*sigma)
 dlogcdt <- -tau*w*((1/(w-1))+((cosh(2*omega))/(w*cosh(2*omega)+1)))
 dlogcdt <- dlogcdt+((nu*w*sinh(2*omega))/(w*cosh(2*omega)+1))
    dzdt <- -(z+w^(.5)*sinh(omega))*dlogcdt-tau*w^(.5)*sinh(omega)+nu*w^(.5)*cosh(omega)  
    dldt <- -dlogcdt-(1/tau)-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdt+(r*(r+nu))/tau    
 d2ldmdt <- -(dldm*dldt)
                       },
  d2ldddv = function() {
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
   dldd <- (z+w^(0.5)*sinh(omega))*(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))
   dldd <- (dldd-1)/sigma       
dlogcdv <- (tau*w*sinh(2*omega))/(w*cosh(2*omega)+1)
   dzdv <- -(z+w^(.5)*sinh(omega))*dlogcdv+(tau*w^(.5)*cosh(omega))
   dldv <- -dlogcdv-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdv+r         
d2ldddv <- -(dldd*dldv)
                       },
  d2ldddt = function() {
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
   dldd <- (z+w^(0.5)*sinh(omega))*(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))
   dldd <- (dldd-1)/sigma     
dlogcdt <- -tau*w*((1/(w-1))+((cosh(2*omega))/(w*cosh(2*omega)+1)))
dlogcdt <- dlogcdt+((nu*w*sinh(2*omega))/(w*cosh(2*omega)+1))
   dzdt <- -(z+w^(.5)*sinh(omega))*dlogcdt-tau*w^(.5)*sinh(omega)+nu*w^(.5)*cosh(omega)  
   dldt <- -dlogcdt-(1/tau)-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdt+(r*(r+nu))/tau    
d2ldddt <- -(dldd*dldt)  
                       },
  d2ldvdt = function() {
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
dlogcdv <- (tau*w*sinh(2*omega))/(w*cosh(2*omega)+1)
   dzdv <- -(z+w^(.5)*sinh(omega))*dlogcdv+(tau*w^(.5)*cosh(omega))
   dldv <- -dlogcdv-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdv+r
dlogcdt <- -tau*w*((1/(w-1))+((cosh(2*omega))/(w*cosh(2*omega)+1)))
dlogcdt <- dlogcdt+((nu*w*sinh(2*omega))/(w*cosh(2*omega)+1))
   dzdt <- -(z+w^(.5)*sinh(omega))*dlogcdt-tau*w^(.5)*sinh(omega)+nu*w^(.5)*cosh(omega)  
   dldt <- -dlogcdt-(1/tau)-(z/(z*z+1)+(r/(tau*(z*z+1)^(.5))))*dzdt+(r*(r+nu))/tau          
d2ldvdt <- -(dldv*dldt)  
                       },
 G.dev.incr  = function(y,mu,sigma,nu,tau,...) 
                       { 
G.dev.incr <- -2*dJSU(y,mu,sigma,nu,tau,log=TRUE)
                        } ,                     
         rqres = expression(   
               rqres(pfun="pJSU", type="Continuous", y=y, mu=mu, sigma=sigma, nu=nu, tau=tau)) ,
    mu.initial = expression(mu <- (y+mean(y))/2),# rep(mean(y),length(y)) 
 sigma.initial = expression(sigma<- rep(sd(y)/5, length(y))),
    nu.initial = expression(nu <- rep(0, length(y))), 
   tau.initial = expression(tau <-rep(.5, length(y))), 
      mu.valid = function(mu) TRUE, 
   sigma.valid = function(sigma)  all(sigma > 0),
      nu.valid = function(nu) TRUE , 
     tau.valid = function(tau) all(tau > 0), 
       y.valid = function(y)  TRUE
          ),
            class = c("gamlss.family","family"))
}
#----------------------------------------------------------------------------------------
dJSU <- function(y, mu = 0, sigma = 1, nu = 1, tau = .5, log = FALSE)
 {
          if (any(sigma < 0))  stop(paste("sigma must be positive", "\n", "")) 
          if (any(tau < 0))  stop(paste("tau must be positive", "\n", ""))  
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (y-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau
 loglik <- -log(sigma)-log(c)-log(tau)-.5*log(z*z+1)-.5*log(2*pi)-.5*r*r
       if(log==FALSE) ft  <- exp(loglik) else ft <- loglik 
       ft
  }    
#----------------------------------------------------------------------------------------  
pJSU <- function(q, mu = 0, sigma = 1, nu = 1, tau = .5, lower.tail = TRUE, log.p = FALSE)
 {  
         if (any(sigma < 0))  stop(paste("sigma must be positive", "\n", "")) 
         if (any(tau < 0))  stop(paste("tau must be positive", "\n", ""))           
      w <- exp(tau^2)
  omega <- -nu*tau 
      c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)  
      z <- (q-(mu+c*sigma*w^(.5)*sinh(omega)))/(c*sigma)
      r <- -nu + asinh(z)/tau    
      p <- pNO(r,0,1)
      if(lower.tail==TRUE) p  <- p else  p <- 1-p 
      if(log.p==FALSE) p  <- p else  p <- log(p) 
      p
 }
#----------------------------------------------------------------------------------------  
qJSU <-  function(p, mu=0, sigma=1, nu=0, tau=.5, lower.tail = TRUE, log.p = FALSE)
 {   
    if (any(sigma < 0))  stop(paste("sigma must be positive", "\n", "")) 
    if (any(tau < 0))  stop(paste("tau must be positive", "\n", ""))  
    if (log.p==TRUE) p <- exp(p) else p <- p
    if (any(p <= 0)|any(p >= 1))  stop(paste("p must be between 0 and 1", "\n", ""))       
    if (lower.tail==TRUE) p <- p else p <- 1-p
    r <- qNO(p,0,1)
    z <- sinh(tau*(r+nu))
    w <- exp(tau^2)
omega <- -nu*tau 
    c <- (.5*(w-1)*(w*cosh(2*omega)+1))^(-0.5)     
    q <- (mu+c*sigma*w^(.5)*sinh(omega))+c*sigma*z   
    q
 }
#-----------------------------------------------------------------  
rJSU <- function(n, mu=0, sigma=1, nu=0, tau=.5)
  {
    if (any(sigma <= 0))  stop(paste("sigma must be positive", "\n", "")) 
    n <- ceiling(n)
    p <- runif(n)
    r <- qJSU(p,mu=mu,sigma=sigma,nu=nu,tau=tau)
    r
  }
#-----------------------------------------------------------------  
