\name{NO.var}

\alias{NO.var}
\alias{dNO.var}
\alias{pNO.var}
\alias{qNO.var}
\alias{rNO.var}


\title{Normal distribution (with variance as sigma parameter) for fitting a GAMLSS}
\description{
The function \code{NO.var()} defines the normal distribution, a two parameter distribution, for a 
\code{gamlss.family} object to be used in GAMLSS fitting 
using the function \code{gamlss()} with mean equal to \code{mu}
and variance equal to \code{sigma}.  
The functions \code{dNO.var}, \code{pNO.var}, \code{qNO.var} and \code{rNO.var} define the density, distribution function, quantile function and random
generation for this specific parameterization of the normal distribution. 
 
[A alternative parameterization with \code{sigma} as the standard deviation is given in the function \code{NO()}]   
}
\usage{
NO.var(mu.link = "identity", sigma.link = "log")
dNO.var(y, mu = 0, sigma = 1, log = FALSE)
pNO.var(q, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)
qNO.var(p, mu = 0, sigma = 1, lower.tail = TRUE, log.p = FALSE)
rNO.var(n, mu = 0, sigma = 1)
}

\arguments{
  \item{mu.link}{ Defines the \code{mu.link}, with "identity" link as the default for the mu parameter}
  \item{sigma.link}{Defines the  \code{sigma.link}, with "log" link as the default for the sigma parameter}
   \item{y,q}{vector of quantiles}
  \item{mu}{vector of location parameter values}
  \item{sigma}{vector of scale parameter values}
  \item{log, log.p}{ logical; if TRUE, probabilities p are given as log(p).}
   \item{lower.tail}{logical; if TRUE (default), probabilities are P[X <= x],
          otherwise, P[X > x] }
  \item{p}{vector of probabilities. }
  \item{n}{ number of observations. If \code{length(n) > 1}, the length is
          taken to be the number required}
  }
\details{
  The parametrization of the normal distribution given in the function \code{NO.var()} is 
 \deqn{f(y|\mu,\sigma)=\frac{1}{\sqrt{2 \pi \sigma}}\exp\left[-\frac{1}{2}\frac{(y-\mu)^2}{\sigma}\right]}{f(y|mu,sigma)=(1/(sqrt(2*pi*sigma))* exp(-0.5*(y-mu)^2/sigma)} 
 
 for \eqn{y=(-\infty,\infty)}{y=(-Inf,+Inf)}, \eqn{\mu=(-\infty,+\infty)}{\mu=(-Inf,+Inf)} and \eqn{\sigma>0}. 
}
\value{
  returns a \code{gamlss.family} object which can be used to fit a normal distribution in the \code{gamlss()} function. 
    
}
\references{
Rigby, R. A. and  Stasinopoulos D. M. (2005). Generalized additive models for location, scale and shape,(with discussion), 
\emph{Appl. Statist.}, \bold{54}, part 3, pp 507-554.

Stasinopoulos D. M., Rigby R.A. and Akantziliotou C. (2006) Instructions on how to use the GAMLSS package in R.
Accompanying documentation in the current GAMLSS  help files, (see also  \url{http://www.gamlss.com/}).

 }

\author{Mikis Stasinopoulos, Bob Rigby and Calliope Akantziliotou}

\note{ For the function \code{NO()}, \eqn{\mu}{mu} is the mean and \eqn{\sigma}{sigma} is the standard deviation (not the variance) of the normal distribution. 
       [The function \code{NO.var()} defines the normal distribution with \eqn{\sigma}{sigma} as the variance.]}



\seealso{ \code{\link{gamlss}}, \code{\link{gamlss.family}}}

\examples{
NO()# gives information about the default links for the normal distribution
dat<-rNO(100)        
gamlss(dat~1,family=NO) # fits a constant for mu and sigma 
plot(function(y) dNO(y, mu=10 ,sigma=2), 0, 20)
plot(function(y) pNO(y, mu=10 ,sigma=2), 0, 20)
plot(function(y) qNO(y, mu=10 ,sigma=2), 0, 1)

}
\keyword{distribution}
\keyword{regression}% 
