\name{VGD}
\alias{VGD}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Validation global deviance}
\description{
 This function helps to validate a GAMLSS model by randomly splitting the data into training
(around 60\%) and validation (around 40\%) sets. It minimizes the global deviance for the
training data set and then uses the validation set to calculate the global deviance.
The resulting  validation global deviance (VGD) can be used for selecting the
distribution, the terms in the model or a degrees of freedom for smoothing.
}
\usage{
VGD(formula = NULL, sigma.fo = ~1, nu.fo = ~1, tau.fo = ~1, 
    data = NULL, family = NO, 
    control = gamlss.control(trace = FALSE), rand = NULL, ...)
}
\arguments{
  \item{formula}{ a gamlss formula for \code{mu} (including the response on the left)}
  \item{sigma.fo}{a formula for  \code{sigma} }
  \item{nu.fo}{ a formula for  \code{nu} }
  \item{tau.fo}{ a formula for  \code{tau} }
  \item{data}{the data set used for the fitting}
  \item{family}{a \code{\link{gamlss.family}} object }
  \item{control}{\code{gamlss.control} to be passed to \code{gamlss}}
  \item{rand}{a random vector of one and two indicating whether is the trainining set (1) or the 
    validation set (2) i.e. created in advance using something like 
      \code{rand <- sample(2, N, replace=T, prob=c(0.6,0.4))} where N is the length of the data}
  \item{\dots}{ for extra arguments to be passed in the \code{gamlss} fit }
}
\details{
}
\value{
 Rerurns a validated global deviance
}
\references{Rigby, R. A. and  Stasinopoulos D. M. (2005). Generalized additive models for location, scale and shape,(with discussion), 
\emph{Appl. Statist.}, \bold{54}, part 3, pp 507-554.

Stasinopoulos D. M., Rigby R.A. and Akantziliotou C. (2006) Instructions on how to use the GAMLSS package in R.
Accompanying documentation in the current GAMLSS  help files, (see also  \url{http://www.gamlss.com/}).}
\author{ Mikis Stasinopoulos \email{d.stasinopoulos@londonmet.ac.uk} and Bob Rigby \email{r.rigby@londonmet.ac.uk}  }
\note{
}
\seealso{\code{\link{gamlss.family}}, \code{\link{gamlss}},  \code{\link{deviance.gamlss}}}
\examples{
data(abdom)
# generate the random split of the data
rand <- sample(2, 610, replace=TRUE, prob=c(0.6,0.4))
# the proportions in the sample
table(rand)/610
VGD(y~cs(x,df=2),sigma.fo=~cs(x,df=1), data=abdom, family=LO, rand=rand)
}
\keyword{regression}% 
