\name{fitted.plot}
\alias{fitted.plot}

\title{Plots The Fitted Values of a GAMLSS Model}
\description{
 This function, applicable only to a models with a single explanatory variable, plots the fitted values for all the parameters
  of a GAMLSS model against the (one) explanatory variable. It is also useful for comparing the fits for more than one model.  
}
\usage{
fitted.plot(object, ..., x = NULL, color = TRUE, line.type = FALSE)
}

\arguments{
  \item{object}{a fitted GAMLSS model object(with only one explanatory variable)}
  \item{\dots}{optionally more fitted GAMLSS model objects }
  \item{x}{The unique explanatory variable }
  \item{color}{whether the fitted lines plots are shown in colour, \code{color=TRUE} (the default) or not \code{color=FALSE}}
  \item{line.type}{whether the line type should be different or not. The default is  \code{color=FALSE}}
}

\value{
 A plot of the fitted values against the explanatory variable 
}
\references{Rigby, R. A. and  Stasinopoulos D. M. (2005). Generalized additive models for location, scale and shape,(with discussion), 
\emph{Appl. Statist.}, \bold{54}, part 3, pp 507-554.

Stasinopoulos D. M., Rigby R.A. and Akantziliotou C. (2006) Instructions on how to use the GAMLSS package in R.
Accompanying documentation in the current GAMLSS  help files, (see also  \url{http://www.gamlss.com/}).   }
\author{Mikis Stasinopoulos \email{d.stasinopoulos@londonmet.ac.uk}, Bob Rigby \email{r.rigby@londonmet.ac.uk} and Calliope Akantziliotou }

\seealso{  \code{\link{gamlss}}, \code{\link{centiles}}, \code{\link{centiles.split}} }
\examples{
data(abdom)
h1<-gamlss(y~cs(x,df=3), sigma.formula=~x, family=BCT, data=abdom)
h2<-gamlss(y~cs(x,df=3), sigma.formula=~cs(x,df=3), family=BCT, data=abdom)
fitted.plot(h1,h2,x=abdom$x)
rm(h1,h2)
}
\keyword{regression}% 
