\name{histDist}
\alias{histDist}

\title{This function plots the histogram and a fitted (GAMLSS family) distribution to a variable}
\description{
  This function fits constants to the parameters of a GAMLSS family distribution and them plot the histogram and 
  the fitted distribution. 
}
\usage{
histDist(y, family = NO, freq = NULL, xmin = NULL, 
         xmax = NULL, g.control = gamlss.control(),
         density = FALSE, main = NULL,  ...)
}
\arguments{
  \item{y}{a vector}
  \item{family}{a continuous GAMLSS family distribution}
  \item{freq}{ the frequencies of the data in \code{y} if exist. \code{freq} is used as \code{weights} in the \code{gamlss} fit}
  \item{xmin}{the minimum x-variable value (if the default values are out of range)}
  \item{xmax}{the maximum x-variable value (if the default values are out of range)}
  \item{g.control}{this is similar to \code{gamlss.control} in case that some of the control parameters have to be changed }
 \item{density}{default value is FALSE. Change to TRUE if you would like a non-parametric density plot together with the parametric 
                fitted distribution plot (for continuous variable only)} 
 \item{main}{the main title for the plot}
 \item{\dots}{for extra arguments to be passed to the \code{truehist()} or the \code{gamlss} function}
}
\details{
  This function first fits constants for each parameters of a GAMLSS distribution family using the \code{gamlss} function  
  and them plots the fitted distribution together with the appropriate plot according to whether 
  the \code{y} variable is of a continuous or discrete type. Histogram is plotted for continuous and barplot for discrete variables. 
  The function \code{truehist} of 
  Venables and Ripley's MASS package is used for the histogram plotting.  
}
\value{
  returns a plot
}
\references{Rigby, R. A. and  Stasinopoulos D. M. (2005). Generalized additive models for location, scale and shape,(with discussion), 
\emph{Appl. Statist.}, \bold{54}, part 3, pp 507-554.

Stasinopoulos D. M., Rigby R.A. and Akantziliotou C. (2006) Instructions on how to use the GAMLSS package in R.
Accompanying documentation in the current GAMLSS  help files, (see also  \url{http://www.gamlss.com/}).  }
\author{Mikis Stasinopoulos}

\seealso{\code{\link{gamlss}}, \code{\link{gamlss.family}} }
\examples{
data(abdom)
attach(abdom)
histDist(y,family="NO")
# use the ymax argument of the of the truehist()
histDist(y,family="NO",ymax=0.005)
# bad fit use PE
histDist(y,family="PE",ymax=0.005)
detach(abdom)
# discere data counts 
#  Hand at al. p150  Leptinotarsa decemlineata
y <- c(0,1,2,3,4,6,7,8,10,11)
freq <- c(33,12,5,6,5,2,2,2,1,2) 
histDist(y, "NBI", freq=freq)
# the same as 
histDist(rep(y,freq), "NBI")
}
\keyword{regression}% 
