\name{ps}
\alias{ps}

\title{ Specify a Penalised Spline Fit in a GAMLSS Formula}
\description{ The function takes a vector and returns it with several attributes. 
The vector is used in the construction of the model matrix.  The function \code{ps()} does no smoothing, 
but assigns the attributes to the vector to aid gamlss in the smoothing.
The function doing the smoothing is \code{\link{gamlss.cs}()} (a modified version of the R function 
   \code{smooth.spline()}) which is used at the backfitting function \code{\link{additive.fit}} .
}
\usage{
ps(x, df = 3, lambda = NULL, ps.intervals = 20, degree = 3, order = 3)
}


\arguments{
  \item{x}{the univariate predictor, (or expression, that evaluates to a numeric vector).}
  \item{df}{the desired equivalent number of degrees of freedom (trace of the smoother matrix minus two for the constant and linear fit }
  \item{lambda}{the smoothing parameter, }
  \item{ps.intervals}{the no of break points in the x-axis}
  \item{degree}{the degree of the piecewise polynomial}
  \item{order}{the difference in the coefficients}
} 
\details{
The \code{ps()} function is based on Brian Marx function which can be found in \url{http://www.stat.lsu.edu/faculty/marx/} 
}

\value{
  
 the vector x is returned, endowed with a number of attributes. The vector itself is used in the construction of the model matrix, 
  while the attributes are needed for the backfitting algorithms \code{additive.fit()}.  
}
\references{

\url{http://www.stat.lsu.edu/faculty/marx/} 

Eilers, P. H. C. and Marx, B. D. (1996). Flexible smoothing with
B-splines and penalties (with comments and rejoinder). \emph{Statist. Sci},
\bold{11}, 89-121.

Rigby, R. A. and  Stasinopoulos D. M. (2005). Generalized additive models for location, scale and shape,(with discussion), 
\emph{Appl. Statist.}, \bold{54}, part 3, pp 507-554.
}
\author{ Mikis Stasinopoulos \email{d.stasinopoulos@londonmet.ac.uk}, Bob Rigby \email{r.rigby@londonmet.ac.uk }}
\note{ }


\seealso{ \code{\link{gamlss}}, \code{\link{gamlss.ps}}, \code{\link{cs}}}
\examples{
data(aids)
attach(aids)
# fitting a smoothing cubic spline with 7 degrees of freedom
# plus the a quarterly  effect  
aids1<-gamlss(y~ps(x,df=7)+qrt,data=aids,family=PO) # 
plot(x,y)
lines(x,fitted(aids1))
rm(aids1)
detach(aids)
}
\keyword{regression}% 
